import logging

import pymysql
import pytest
from pytest_rmysql.config import parse_config


def pytest_addoption(parser):
    """plugin configuration in .ini"""
    parser.addini(name='rmysql_host', help="MySQL host", default='localhost')

    parser.addini(name='rmysql_port', default=3306, help="MySQL port")

    parser.addini(name='rmysql_database', help="MySQL database name", default=None)

    parser.addini(name='rmysql_username', help="MySQL user name", default=None)

    parser.addini(name='rmysql_password', help="MySQL password", default=None)

    # parser.addini(name='rmysql_params',  help="MySQL parameters", default="")

    parser.addini(name='rmysql_charset', help="MySQL charset", default="utf8")

    """plugin configuration in cmd"""
    group = parser.getgroup('rmysql', 'RMySQL plugin')
    group.addoption(
        "--rmysql-host",
        type=str,
        default="localhost",
        help="MySQL host"
    )
    group.addoption(
        "--rmysql-port",
        type=int,
        default=3306,
        help="MySQL port"
    )
    group.addoption(
        "--rmysql-username",
        type=str,
        default="root",
        help="MySQL username"
    )
    group.addoption(
        "--rmysql-password",
        type=str,
        default=None,
        help="MySQL password"
    )
    group.addoption(
        "--rmysql-database",
        type=str,
        default=None,
        help="MySQL database name"
    )
    # group.addoption(
    #     "--rmysql-params",
    #     type=str,
    #     default="",
    #     help="MySQL parameters"
    # )
    group.addoption(
        "--rmysql-charset",
        type=str,
        default="utf8",
        help="MySQL charset"
    )

@pytest.fixture(scope='session')
def rmysql(request):
    config = parse_config(request)
    logger = logging.getLogger()
    logger.info(config)
    mysql_conn = pymysql.connect(
        host=config.get('host'),
        port=config.get('port'),
        user=config.get('username'),
        password=config.get('password'),
        database=config.get('database'),
        charset=config.get('charset'),
    )
    mysql_conn.connect()
    mysql_conn.cursor().execute("select 1")

    yield mysql_conn

    mysql_conn.close()


from typing import TypedDict, Any
from _pytest.fixtures import FixtureRequest


class MySQLConfig(TypedDict):
    host: str
    port: int
    database: str
    username: str
    password: str
    # params: str
    charset: str

def parse_config(request: FixtureRequest) -> MySQLConfig:
    def get_option(option: str) -> Any:
        option_name = "rmysql_" + option
        if option == "host":
            if request.config.getoption(option_name) == "localhost":
                return request.config.getini(option_name)
            elif request.config.getini(option_name) == "localhost":
                return request.config.getoption(option_name)
        elif option == "port":
            if request.config.getoption(option_name) == 3306:
                return request.config.getini(option_name)
            elif request.config.getini(option_name) == 3306:
                return request.config.getoption(option_name)
        elif option == "username":
            if request.config.getoption(option_name) == "root":
                return request.config.getini(option_name)
            elif request.config.getini(option_name) == "root":
                return request.config.getoption(option_name)
        elif option == "charset":
            if request.config.getoption(option_name) == "utf8":
                return request.config.getini(option_name)
            elif request.config.getini(option_name) == "utf8":
                return request.config.getoption(option_name)
        return request.config.getoption(option_name) or request.config.getini(option_name)

    config : MySQLConfig = {
        'host': get_option('host'),
        'port': get_option('port'),
        'database': get_option('database'),
        'username': get_option('username'),
        'password': get_option('password'),
        # 'params': get_option('params'),
        'charset': get_option('charset'),
    }

    return config